#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
django-celery-growthmonitor, a Django helper to monitor jobs running Celery tasks.
Copyright (C) 2017  Marc BOURQUI

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""

__license__ = 'GNU GPLv3 <https://www.gnu.org/licenses/gpl-3.0.html>'
__author__ = 'Marc Bourqui <pypi.kemar@bourqui.org>'
__version__ = '0.0.0.dev30'
__version_info__ = tuple([int(num) if num.isdigit() else num for num in __version__.replace('-', '.', 1).split('.')])
__status__ = 'Beta'



from celery.canvas import chain as celery_chain

from celery_growthmonitor.models import MetaJob
from .tasks import start, stop, remove_old_jobs


def chain(metajob, *tasks):
    """
    Build a chain of tasks, adding monitoring and maintenance tasks at the beginning and end of the chain

    Parameters
    ----------
    metajob : MetaJob
    tasks : celery.shared_task

    Returns
    -------
    celery.canvas.chain

    """
    try:
        flow = eval("""(
            start.s(metajob),
            *tasks,
            stop.s(),
            remove_old_jobs.s(),
        )""")
    except SyntaxError:
        # Python < 3.5
        flow = (start.s(metajob),)
        flow += tuple([task for task in tasks])
        flow += (
            stop.s(),
            remove_old_jobs.s(),
        )
    return celery_chain(*flow)



from collections import namedtuple

from celery import shared_task

from ..models.metajob import MetaJob

ReturnTuple = namedtuple('ReturnTuple', ['metajob', 'results'])


def _compat_return(metajob: MetaJob, *args):
    return ReturnTuple(metajob, args)


def extract_metajob(previous_task_results, *args):
    """

    Parameters
    ----------
    previous_task_results
    args

    Returns
    -------
    ReturnTuple
        metajob : MetaJob
            post_serialization() has already been called
        results : tuple

    """
    if isinstance(previous_task_results, ReturnTuple):
        return _compat_return(previous_task_results.metajob.post_serialization(),
                              *(previous_task_results.results + args))
    elif isinstance(previous_task_results, tuple):
        return _compat_return(previous_task_results[0].post_serialization()
                              , *(previous_task_results[1:] + args))
    return _compat_return(previous_task_results.post_serialization(), *args)


# ==================================================
#   MONITORING TASKS
# ==================================================

@shared_task
def start(metajob):
    """
    Having a task for starting the job makes sure we measure the right time of running

    Parameters
    ----------
    metajob : MetaJob

    Returns
    -------
    MetaJob

    """
    metajob.post_serialization()
    metajob.start()
    return metajob.pre_serialization()


@shared_task
def stop(metajob, *args):
    """
    Having a task for stopping the job makes sure we measure the right time of completion (previous task is obviously
    done)

    Parameters
    ----------
    metajob : MetaJob or tuple
    args

    Returns
    -------
    MetaJob

    """
    metajob, args = extract_metajob(metajob, *args)
    metajob.stop()
    return _compat_return(metajob.pre_serialization(), *args)


# ==================================================
#   MAINTENANCE TASKS
# ==================================================

@shared_task
def remove_old_jobs(metajob, *args):
    """

    Parameters
    ----------
    metajob : MetaJob or tuple
    args

    Returns
    -------

    """
    metajob, args = extract_metajob(metajob, *args)
    from django.utils.timezone import now
    candidates = metajob.job.__class__.objects.filter(closure__lt=now())
    for candidate in candidates:
        candidate.delete()
    return _compat_return(metajob.pre_serialization(), *args)
